#ifndef cDAG_TYPES_HPP
#define cDAG_TYPES_HPP

#include <list>
#include <mpi.h>
#include <vector>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cDAG_Graph.hpp"

#define cDAG_SUCCESS 0

#define TEST_VEC  // vector test with testany
//#define TEST_LIST // list test with test

#ifdef TRAN_OF
#include "../of/of.h"
#endif

#ifndef TRAN_MPI
#ifndef TRAN_OF
#define TRAN_MPI
#endif
#endif

#ifdef TRAN_MPI
typedef MPI_Request sched_req;
#endif
#ifdef TRAN_OF
typedef OF_Request sched_req;
#endif


typedef int cDAG_Fid;
typedef void* cDAG_Func;
typedef int cDAG_Vertex;
typedef int cDAG_Opcode;
const cDAG_Vertex cDAG_VERTEX_NULL=-1;
typedef int cDAG_MemType;

const int cDAG_USERSPACE=10;
const int cDAG_SCRATCHPAD=11;

typedef struct {
  char* active;
  char* copy;
  unsigned int size;
} cDAG_Schedule; // the user schedule pointer

typedef char* schedule_t; // an actual schedule

#if 0
/* used to hang off a communicator */
typedef struct {
  MPI_Comm mycomm; /* save the shadow communicator here */
  int tag;
} NBC_Comminfo;
#endif

typedef struct {
//  MPI_Comm comm;
//  MPI_Comm mycomm;
  int tag;
  enum {NEW, RUNNING} state;
//  NBC_Comminfo *comminfo;
#ifdef TEST_LIST
  std::list<std::pair<sched_req,int> > *reqs;
#endif 
#ifdef TEST_VEC
  std::vector<int> *offsets;
  std::vector<sched_req> *reqs;
#endif
  pthread_mutex_t lock;
  schedule_t schedule; // the active schedule (associated with this handle)
  void* tmpbuf; /* temporary buffer e.g. used for Reduce */
/* TODO: we should make a handle pointer to a state later (that the user
 * can move request handles) */
} cDAG_Handle_struct;
typedef cDAG_Handle_struct* cDAG_Handle;

#endif
